/*
 * HP8673E device support
 */

#include <epicsStdio.h>
#include <devCommonGpib.h>
#include <math.h>
#include <stdlib.h>

/*
 * Names to be used for the dset tables.   
 */
#define DSET_AI     devAiHP8673E
#define DSET_AO     devAoHP8673E

#include <devGpib.h> /* must be included after DSET defines */

#define TIMEOUT     2.0    /* I/O must complete within this time */
#define TIMEWINDOW  2.0    /* Wait this long after device timeout */

/*
 * Custom conversion routine for power level
 */
static int
convertPowerLevel(struct gpibDpvt *pdpvt, int P1, int P2, char **P3)
{
    struct aoRecord *pao = ((struct aoRecord *)(pdpvt->precord));
    char sign;
    int scale;
    double v = fabs(pao->val);
    double vernier;

    sign = (pao->val >= 0) ? '+' : '-';
    scale = (int)v / 10;
    vernier = v - (scale * 10);
    sprintf(pdpvt->msg, "PL %c%d %g DM", sign, scale, vernier);
    return 0;
}

/*
 * Command table
 */
static struct gpibCmd gpibCmds[] = {
    /* Param 0 -- Set power level */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, NULL, 0, 200, convertPowerLevel, 0, 0, NULL, NULL, NULL},

    /* Param 1 - Set frequency */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "FR %.0f MZ", 0, 200, NULL, 0, 0, NULL, NULL, NULL},
};

/*
 * Initialize device support parameters
 */
static long init_ai(int parm)
{
    if(parm==0) {
        devSupParms.name = "devHP8673E";
        devSupParms.gpibCmds = gpibCmds;
        devSupParms.numparams = sizeof(gpibCmds)/sizeof(struct gpibCmd);
        devSupParms.timeout = TIMEOUT;
        devSupParms.timeWindow = TIMEWINDOW;
        devSupParms.respond2Writes = -1;
    }
    return(0);
}
